#!/usr/bin/env python
import sys
import os
import dfont
from dttcmd import *

#----------------------------------------
#   var
#----------------------------------------
txt = ""

#func_arg_list: [ [arg_count, retvat_count], ...]
#DETERMINE FROM AUTO-INSTRUCTION BY:
# 1.LOOK IN INSTRUCTION CODE TO DETERMINE arg_count
# 2.LOOK IN fpgm CODE TO DETERMINE retval_count
func_arg_list = [ [2,0], [1,0], [3,1], [1,1], [4,0], [0,1], [0,1], [0,1], [3,0] ]


cvt_dict = {
    'cvt_0'     : 0,   #0
    'vheight'   : 1,   #417
    'vheight_cap': 2,   #563
    'vheight_po': 3,   #603
    'cvt_4'     : 4,   #43
    'hstem'     : 5,   #41
    "stem"      : 5,      #41
    'vstem'     : 6,   #41
    'cvt_7'     : 7,   #39
    'cvt_8'     : 8,   #37
    'cvt_9'     : 9,   #46
    'cvt_10'    : 10,   #24
    'cvt_11'    : 11,   #33
    'cvt_12'    : 12,   #35
    'cvt_13'    : 13,   #29
    'cvt_14'    : 14,   #49
    'vstem_thick'       : 15,   #50
    'hstem_thick'       : 16,   #60
    "stem_sara"         : 17,   #36
    "vheight_cap_shoot" : 18,   #576
    "vbase_shoot"       : 19,   #16
    "vbase_thai_shoot"  : 20,   #10
    "vheight_shoot"     : 21,   #431
    "vheight_lo"        : 22,   #186
    "vheight_thai"      : 23,   #445
    "vheight_shoot_thai": 24,   #455
    "vheight_lu"        : 25,   #165
    "vheight_num"       : 26,   #366
    "vheight_num_shoot" : 27,   #376
    "headdia"           : 28,   #160
    "head_sara"         : 29,   #120
    "w_422"             : 30,   #422
    "w_356"             : 31,   #356
    "w_275"             : 32,   #275
}


#----------------------------------------
#   code
#----------------------------------------
def gedit(txt,gen_pseudo=False):
    """invoke gedit"""
    os.system('killall gedit')
    if gen_pseudo:
        f = open('xcode.txt','w')
        #strip blank line
        lt = [i.strip() for i in txt.split('\n')]
        txt = '\n'.join([i for i in lt if i])
        f.write(txt)
        f.close()
        os.system('gedit xcode.txt &')
    f = open('x.txt','w')
    f.write('\n'.join(dfont.inst_encode(txt, cvt_dict, func_arg_list )))
    f.close()
    os.system('gedit x.txt &')
    return


#----------------------------------------
dh_dstr_1 = "15+4 16+3 17+3 18+3 "
dh_dstr_2 = "15-4 16-3 17-3 18-3 "

def dh1(*l): #dry head
    d = {}
    for s in l:
        d[s-1] = dh_dstr_1
        d[s+1] = dh_dstr_2
    return f_dstr_dict(d)
#    return f_dstr_dict({
#        s-1: "15+4 16+3 17+3 18+3 ",
#        s+1: "15-4 16-3 17-3 18-3 " })

def dh2(*l): #dry head
    d = {}
    for s in l:
        d[s-1] = dh_dstr_2
        d[s+1] = dh_dstr_1
    return f_dstr_dict(d)
#    return f_dstr_dict({
#        s-1: "15-4 16-3 17-3 18-3 ",
#        s+1: "15+4 16+3 17+3 18+3 " })

#----------------------------------------
def dx(s,x):
    return """
SHPIX   %s %s
""" % (x,s)

#----------------------------------------

ds_hstem_1 = "9-2 10-2 11-2 12-1 "
ds_hstem_2 = "9+3 10+3 11+2 12+1 "

def hstem1(s):
    return mirpmg(s,"stem",ds_hstem_1)

def hstem2(s):
    return mirpmg(s,"stem",ds_hstem_2)

def hstem1m(s):
    return mirp0mg(s,"stem",ds_hstem_1)

def hstem2m(s):
    return mirp0mg(s,"stem",ds_hstem_2)

def vstem1(s,sft=-12):
    return pl(""
        , mirpmrg(s,"stem",ds_hstem_1)
        , dx(s,sft)
        )

def vstem2(s):
    return mirpmrg(s,"stem",ds_hstem_2)

def vstem1m(s):
    return mirp0mrg(s,"stem",ds_hstem_1)

def vstem2m(s):
    return mirp0mrg(s,"stem",ds_hstem_2)

ds_hstem_sara_1 = "9-3 10-3 11-3 12-2 13-1 "
ds_hstem_sara_2 = "9+4 10+4 11+3 12+2 13+1 "

def hstemsara1(s):
    return mirpmrg(s,"stem_sara",ds_hstem_sara_1)

def hstemsara1m(s):
    return mirp0mrg(s,"stem_sara",ds_hstem_sara_1)

def hstemsara2(s):
    return mirpmrg(s,"stem_sara",ds_hstem_sara_2)

def hstemsara2m(s):
    return mirp0mrg(s,"stem_sara",ds_hstem_sara_2)

def vstemsara1(s):
    return mirpmrg(s,"stem_sara",ds_hstem_sara_1)

def vstemsara1m(s):
    return mirp0mrg(s,"stem_sara",ds_hstem_sara_1)

def vstemsara2(s):
    return mirpmrg(s,"stem_sara",ds_hstem_sara_2)

def vstemsara2m(s):
    return mirp0mrg(s,"stem_sara",ds_hstem_sara_2)


#================================================
def vbc1(s,li=[]):
    return pl(""
        , mirp0rg(s,"vbase_shoot")
        , f_ip(li)
        )

def vbct(s,li=[]):
    return pl(""
        , mirp0rg(s,"vbase_thai_shoot")
        , f_ip(li)
        )

def vblu(s,li=[]):
    return pl(""
        , mirprg(s,"vheight_lu")
        , f_ip(li)
        )

def vblum(s,li=[]):
    return pl(""
        , mirp0rg(s,"vheight_lu")
        , f_ip(li)
        )

def vtc2(s,li=[]):
    return pl(""
        , mirp0rg(s,"vheight_shoot","15+8 ")
        , f_ip(li)
        )

def vt2(s,li=[]):
    return pl(""
        , mirprg(s,"vheight","13+8 15+8 20+8 25+8 27+8 29+8 32+8 34+8 36+8 ")
        , f_ip(li)
        )

def vt2m(s,li=[]):
    return pl(""
        , mirp0rg(s,"vheight","13+8 15+8 20+8 25+8 27+8 29+8 32+8 34+8 36+8 ")
        , f_ip(li)
        )

def vtpo(s,li=[]):
    return pl(""
        , mirprg(s,"vheight_po")
        , f_ip(li)
        )

def vtpom(s,li=[]):
    return pl(""
        , mirp0rg(s,"vheight_po")
        , f_ip(li)
        )

def vtn(s, li=[]):
    return pl(""
        , mirp0rg(s,"vheight_num")
        , f_ip(li)
        )

def vtnc(s, li=[]):
    return pl(""
        , mirp0rg(s,"vheight_num_shoot")
        , f_ip(li)
        )

def head_0(s,li,dir="+",small=False,ss=-1,l1s=-1,l2s=-1,dia="headdia"):
    if small:
        ds = "10-8 11-4 12-4 15+8 22-8 "
    else:
        ds = "9+4 10-2 11-2 12-2 15+8 22-8 "
    if dia == "head_sara":
        ds = "9+4 10+4 11+4 12+4 19+8 20+8 "
    if dir == "-":
        ds = swap_dstr(ds)
        if dia == "head_sara":
            stem1 = hstemsara2(li[1])
            stem2 = hstemsara1(li[0])
        else:
            stem1 = hstem2(li[1])
            stem2 = hstem1(li[0])
    else:
        l1 = li[0]
        l2 = li[1]
        if dia == "head_sara":
            stem1 = hstemsara1(li[0])
            stem2 = hstemsara2(li[1])
        else:
            stem1 = hstem1(li[0])
            stem2 = hstem2(li[1])
    if len(li)>2:
        ip = f_ip(li[2:])
    else:
        ip = ""
    if ss >= 0:
        add_ss = alignrp(ss)
    else:
        add_ss = ""
    if l1s >= 0:
        add_l1s = shprp2(l1s)
    else:
        add_l1s = ""
    if l2s >= 0:
        add_l2s = shprp2(l2s)
    else:
        add_l2s = ""
    return pl(""
        , stem1
        , add_l1s
        , mirp0rg(s,dia,ds)
        , add_ss
        , ip
        , stem2
        , add_l2s
        )

def head1(s,li,small=False,ss=-1,l1s=-1,l2s=-1):
    #left to right
    return head_0(s,li,dir="+",small=small,ss=ss,l1s=l1s,l2s=l2s)

def head2(s,li,small=False,ss=-1,l1s=-1,l2s=-1):
    #right to left
    return head_0(s,li,dir="-",small=small,ss=ss,l1s=l1s,l2s=l2s)

def headsara1(s,li,small=False,ss=-1,l1s=-1,l2s=-1,dia="head_sara"):
    #left to right
    return head_0(s,li,dir="+",small=small,ss=ss,l1s=l1s,l2s=l2s,dia=dia)

def headsara2(s,li,small=False,ss=-1,l1s=-1,l2s=-1,dia="head_sara"):
    #right to left
    return head_0(s,li,dir="-",small=small,ss=ss,l1s=l1s,l2s=l2s,dia=dia)


#================================================
def v_notch_ko_kai(s1,s1s,s2):
    return pl(""
        , mdrp0rg(s1)
        , alignrp(s1s)
        , vstem1(s2)
        , dx(s2,-8)
        )

def v_head_kho_khuat(s1,s1s,li1, s2,s2s, s3,s4, s5,s6,li6 , add1=""):
    return pl(""
        , vtc2(s1, li1)
        , alignrp(s1s)
        , add1
        , vstem2(s2)
        , shprp2(s2s)
        , mdrp0mrg(s3)
        , mdrpmrg(s4,"9-7 10-7 11-7 12-7 13-7 14-7 15-7 16-7 17-7 ")
        , mdrp0rg(s5)
        , head2(s6,li6,small=True)
        
        )

#================================================
def hs1(s1,ds1="9-8 18-8 "):    #start point #1 forward
    #ko_kai, kho_khwai
    return mdap(s1,ds1)

def hs2(s1,ds1="9-8 "):    #start point #2 reverse
    #kho_khai, cho_chan
    return mdap(s1,ds1)


def hw1(s1,li=[]): 
    #make width #1  forward
    #move from front stem of ko_kai to rear stem
    ds1 = "9+8 15+8 18+8 21+8 23+8 "
    return pl(""
        , mirp0rg(s1,"w_356",ds1)
        , f_ip(li)
        )

def hw2(s1,li=[]): 
    #make width #2  reverse
    ds1 = "13-8 15-8 "
    return pl(""
        , mirp0rg(s1,"w_422",ds1)
        , f_ip(li)
        )

def hw3(s1,li=[]): 
    #make width #3 forward
    ds1 = ""
    return pl(""
        , mirp0rg(s1,"w_275",ds1)
        , f_ip(li)
        )

def hf1(s1,s2,s3,li3,s1s=-1,s2s=-1,s3s=-1,l1s=-1,l2s=-1,dstr1="11+8 15+8 20-8 ",add1=""):
    #front h for tho_thahan, po_pan, no_nu, etc.
    if s1s >= 0:
        add1 = pl(alignrp(s1s), add1)
    if s2s >= 0:
        add2 = shprp2(s2s)
    else:
        add2 = ""
    if s3s >= 0:
        add3 = head2(s3,li3,ss=s3s,l1s=l1s,l2s=l2s)
    else:
        add3 = head2(s3,li3)
    return pl(""
        , mdap(s1,dstr1)
        , add1
        , hstem2(s2)
        , add2
        , add3
        )

def hf_ko_kai(s1,s2,t2,s3,s4, add4=""):
    return pl(""
        , hs1(s1)
        , hstem1(s2)
        , mdrprg(t2,"9+4 ")
        , mdrp0rg(s3,"9+8 10+4 ")
        , hstem1(s4)
        , "%s   #add4" % (add4,)
        )

def hf_kho_khai(s1,s2,li, dstr1="9-8 ", add1=""):
    return pl(""
        , mdap(s1,dstr1)
        , "%s   #add1" % (add1,)
        , head1(s2,li,small=True)
        )

def hf_kho_khuat(s1,t1, s2, li, dstr1="9-8 10-8 11-8 12-8 "):
    add1 = hstem1(t1)
    return hf_kho_khai(s1,s2,li,dstr1,add1)

def hf_pho_samphao(s1,s2,t2,s3,s4,s5,li5):
    return pl(""
        , mdap(s1,"11+8 ")
        , hstem2(s2)
        , mdrprg(t2)
        , mdrp0mrg(s3,"11-4 12-4 13-4 ")
        , hstem1(s4)
        , srp0(s1)
        , head2(s5,li5)
        )

def hr_kho_khai_1(s1,s2, s3,s4, s5,s6, add2="",add3="",add4="",add5="",add6=""):
    return pl(""
        , hs2(s1)
        , hstem2(s2)
        , add2
        , mdrp0rg(s3,"9-8 10-8 ")
        , add3
        , hstem1(s4)
        , add4
        , srp0(s1)
        , mdrp0rg(s5,"9-8 10-8 11-8 12-8 19-8 20-8 ")
        , add5
        , hstem2(s6)
        , add6
        )

def hr_kho_khai(s1,s2, s3,s4,t4, s5,s6):
    add2 = add3 = ""
    add4 = mdrpmrg(t4)
    return  hr_kho_khai_1(s1,s2, s3,s4, s5,s6, add2,add3,add4)

def hr_cho_chang(s1,s2,t21,t22, s3,s4,t4, s5,t5,s6,i6):
    add2 = pl(mdrp0rg(t21), hstem2(t22), srp0(s1))
    add3 = ""
    add4 = mdrpmrg(t4)
    add5 = hstemsara1(t5)
    add6 = f_ip([i6])
    return  hr_kho_khai_1(s1,s2, s3,s4, s5,s6, add2,add3,add4,add5,add6)


def hr_tho_nangmontho(s1,s2, s3,s4,s4s, s5,s6, s7,s8):
    add2 = ""
    add3 = f_dstr(s3, "9+8 10+8 14-8 20-8 ")
    add4 = shprp2(s4s)
    add5 = f_dstr(s5, "12+8 16-8 19+8 ")
    add6 = pl(add_ip(s5,s2,[s7]), mdap(s7), hstem1(s8))
    return  hr_kho_khai_1(s1,s2, s3,s4, s5,s6, add2,add3,add4,add5,add6)

def h_kho_khwai(s1,s2, s3,s4,i4, s5,s6,li6, s7,s8, li7=[]):
    return pl(""
        , hs1(s1)
        , hstem1(s2)
        , mdrp0rg(s3,"9+8 10+8 23+8 24+8 ")
        , hstem1(s4)
        , add_ip(s4,s2,[i4])
        , srp0(s1)
        , mdrp0rg(s5,)
        , head1(s6,li6,small=True)
        , srp0(s1)
        , hw1(s7)
        , hstem2(s8)
        , add_ip(s3,s7,li7)
        )

def hr_kho_rakhang(s1,s2,s2s, s3,s3s, s4,s4s, t41,t42, s5,s6):
    add2 = shprp2(s2s)
    add3 = alignrp(s3s)
    add4 = pl(""
        , shprp2(s4s)
        , mdrp0rg(t41,"13+8 14+8 15+8 16+8 ")
        , hstem1(t42)
        )
    return hr_kho_khai_1(s1,s2, s3,s4, s5,s6, add2,add3,add4)

def hr_no_nu(s1,s2, s3,s4,li):
    return pl(""
        , mdap(s1)
        , hstem2(s2)
        , mdrp0rg(s3)
        , head2(s4,li)
        )

#================================================
pseudo_code_dict = {

#    'a':
#        packline(""
#        , mdap(13)
#        , hstem2(31)
#        , shprp2(35)
#        , shprp2(20)
#        , mdrprg(17)
#        , mdrp0rg(26)
#        , mdrprg(6)
#        , hstem1(41)
#        , ix()
#        #y
#        , sy()
#        , mdap(20)
#        , vstem1(14)
#        , vtc2(10)
#        , vstem2(0)
#        , mdrp0rg(29)
#        , vstem2(38)
#        , mdrp0rg(3)
#        , vstem1(8)
#        , srp0(20)
#        , vbc1(23)
#        , vstem1(44)
#        , iy()
#        ),
#
#    'b':      
#        packline(""
#        , mdap(18)
#        , mdrprg(15)
#        , shprp2(22)
#        , hstem1(11)
#        , shprp2(28)
#        , shprp2(0)
#        , mdrp0rg(6)
#        , hstem2(34)
#        , ix()
#        #y
#        , sy()
#        , mdap(12)
#        , vstem1(18)
#        , vtpo(0)
#        , vstem2(19)
#        , srp0(12)
#        , vtc2(3)
#        , vstem2(25)
#        , srp0(12)
#        , vbc1(9)
#        , vstem1(31)
#        , iy()
#        ),  
#
#    'c':
#        packline(""
#        , mdap(18)
#        , hstem2(12)
#        , shprp2(22)
#        , mdrprg(0)
#        , mdrp0rg(7)
#        , hstem1(28)
#        , ix()
#        #y
#        , sy()
#        , mdap(37)
#        , vtc2(10)
#        , vstem2(25)
#        , srp0(37)
#        , vbc1(4)
#        , vstem1(31)
#        , iy()
#        ),
#
#    'd':
#        packline(""
#        , mdap(0)
#        , mdrprg(4)
#        , hstem2m(18)
#        , alignrp(34)
#        , alignrp(7)
#        , mdrprg(22)
#        , srp0(4)
#        , mdrp0rg(13)
#        , hstem1(28)
#        , ix()
#        #y
#        , sy()
#        , mdap(7)
#        , vstem1(1)
#        , vtpo(0)
#        , vstem2(19)
#        , srp0(7)
#        , vtc2(16)
#        , vstem2(25)
#        , srp0(7)
#        , vbc1(10)
#        , vstem1(31)
#        , iy()
#        ),



    'uni0E01':      #KO KAI
        packline(""
        , hf_ko_kai(16,14,11, 19,8)
        , srp0(16)
        , hw1(25)
        , hstem2(1)
        , ix()
        #y
        , sy()
        , mdap(1)
        , alignrp(15)
        , vtc2(22)
        , vstem2(5)
        , v_notch_ko_kai(17,14,8)
        , iy()
        ),

    'uni0E02':    #KHO KHAI
        packline(""
        , hf_kho_khai(26,20,[51,45])
        , hr_kho_khai(41,39, 6,37,3, 32,12)
        , ix()
        , dh2(23)
        #y
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(38)
        , vt2(41)
        , vtc2(29,[7,37,12,32])
        , vstem2(15)
        , mdrp0rg(17,"12+8 ")
        , head2(23,[42,48],small=True)
        , iy()
        , dh2(20)
        ),

    'uni0E03':    #KHO KHUAT
        packline(""
        , hf_kho_khuat(31,20, 25, [58,52])
        , hr_kho_khai(48,46, 6,44,3, 39,12)
        , add_ip(31,39, [34,35,36, 15,16,17])
        , ix()
        , dh1(22)
        , dh2(28)
        #y
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(45)
        , vt2(48)
        , v_head_kho_khuat(36,34,[7,44,12,39], 15,17, 16,35, 22,28, [49,55])
        , iy()
        , dh2(25)
        ),

    'uni0E04':    #KHO KHWAI
        packline(""
        , h_kho_khwai(28,26, 34,8,11, 13,18,[50,44,24], 40,1,[5,37])
        , ix()
        , dh1(15)
        , dh2(21)
        #y
        , sy()
        , mdap(1)
        , alignrp(27)
        , vtc2(37)
        , vstem2(5)
        , mdrp0rw(15,"9-8 10-8 ")
        , head2(21,[41,47,24,13],small=True)
        , dx(24,12)
        , dx(11,-16)
        , iy()
        , dh2(18)
        ),

    'uni0E05':    #KHO KHON
        packline(""
        , h_kho_khwai(30,28, 36,10,13, 15,20,[54,48,26], 44,1,[39,40,41,7,6,5])
        , ix()
        , dh1(17)
        , dh2(23)
        #y
        , sy()
        , mdap(1)
        , alignrp(29)
        , vtc2(41)
        , alignrp(39)
        , vstem2(5)
        , shprp2(7)
        , mdrp0rg(6,"9-8 10-8 11-8 12-8 13-8 14-8 15-8 ")
        , vstem1(40)
        , mdrp0rw(17)
        , head2(23,[45,51,26,15],small=True)
        , dx(26,12)
        , dx(13,-16)
        , iy()
        , dh2(20)
        ),

    'uni0E06':    #KHO RAKHANG
        packline(""
        , hf_kho_khuat(40,29, 34, [68,62])
        , hr_kho_rakhang(58,56,1, 15,74, 53,4, 10,80, 48,21)
        , add_ip(40,48,[43,44,45,24,25,26])
        , ix()
        , dh1(31)
        , dh2(37)
        , dh1(13)
        , dh2(7)
        #y
        , sy()
        , mdap(1)
        , vstem1(56)
        , vt2(58)
        , v_head_kho_khuat(45,43,[21,48,46,53], 24,26, 25,44, 31,37,[59,65])
        , srp0(1)
        , vbct(7)
        , head1(13,[71,77, 15,54,75,3],small=True)
        , iy()
        , dh2(34)
        , dh1(10)
        ),

    'uni0E07':    #NGO NGU
        packline(""
        , mdap(32)
        , hstem2(20)
        , head2(26,[45,39])
        , srp0(32)
        , mdrp0rg(2)
        , hstemsara1(16)
        , srp0(32)
        , mdrp0rg(8,"14+8 ")
        , hstemsara1(13)
        , ix()
        , dh1(29)
        , dh2(23)
        #y
        , sy()
        , mdap(0)
        , vstem1(17)
        , mdrprg(11)
        , vtc2(29)
        , head2(23,[36,42])
        , iy()
        , dh1(26)
        ),

    'uni0E08':    #CHO CHAN
        packline(""
        , hs2(34)
        , hstem2(17)
        , mdrp0rg(13,"25-8 ")
        , alignrp(37)
        , hstem2(1)
        , head2(7,[47,41])
        , srp0(34)
        , hw2(28,[20,31])
        , hstem1(23)
        , ix()
        , dh1(10)
        , dh2(4)
        #y
        , sy()
        , mdap(1)
        , vtc2(31,[37,14])
        , vstem2(20)
        , mdrprg(26)
        , mdrp0rg(10,"9-8 10-8 ")
        , head2(4,[38,44])
        , iy()
        , dh1(7)
        ),

    'uni0E09':    #CHO CHING
        packline(""
        , hr_no_nu(37,16, 41,47,[72,66])
        , srp0(37)
        , mdrp0rg(13,"14-8 ")
        , alignrp(0)
        , hstem2(1)
        , head2(7,[60,54])
        , mdrp0rg(29)
        , hstem1(24)
        , ix()
        , dh1(10)
        , dh2(4)
        , dh2(44)
        #y
        , sy()
        , mdap(0)
        , vstem1(14)
        , vtc2(34)
        , vstem2(20)
        , mdrp0rg(26,"13+8 14+8 15+8 16+8 ")
        , mdrp0rw(10,"9-8 10-8 11-8 ")
        , head2(4,[51,57])
        , srp0(0)
        , vbct(44)
        , head1(16,[63,69,38,49])
        , iy()
        , dh1(7)
        , dh2(41)
        , dh1(47)
        ),

    'uni0E0A':    #CHO CHANG
        packline(""
        , hf_kho_khai(26,20,[70,64])
        , hr_cho_chang(49,51, 41,35, 6,59,3, 54,46, 12,32)
        , ix()
        , dh2(23)
        #y
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(60)
        , vtc2(29,[12,32,54,46,7,59])
        , mdrp0mrg(38)
        , vstemsara2(43)
        , srp0(29)
        , vstem2(15)
        , mdrp0rg(17,"12+8 ")
        , head2(23,[61,67],small=True)
        , iy()
        , dh2(20)
        ),

    'uni0E0B':    #SO SO
        packline(""
        , hf_kho_khuat(31,20, 25, [77,71])
        , hr_cho_chang(56,57, 48,42, 6,66,3, 61,53, 12,39)
        , add_ip(31,61, [34,35,36,15,16,17])
        , ix()
        , dx(53,-12)
        , dh1(22)
        , dh2(28)
        #y
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(67)
        , v_head_kho_khuat(36,34,[12,39,61,53,7,66], 15,17, 16,35, 22,28, [68,74], add1=pl(mdrp0mrg(45),vstemsara2(50),srp0(36)))
        , iy()
        , dh2(25)
        ),

    'uni0E0C':    #CHO_CHOE
        packline(""
        , mdap(39,"9-8 10-8 11-8 12-8 18-8 19-8 30-8 ")
        , hstem1(27)
        , mdrprg(24)
        , mdrp0rg(42)
        , hstem1(21)
        , srp0(39)
        , head1(33,[63,57])
        #
        , mdap(53,"9-8 23+8 ")
        , hstem2(51)
        , shprp2(1)
        , mdrp0rg(48,"10-8 32+8 39+8 47+8 ")
        , hstem2(14)
        , mdrp0rg(11,"9-4 10-4 11-4 12-4 13+4 21+8 54+8 ")
        , head1(5,[75,69])
        , ix()
        , dh1(30)
        , dh2(36,8)
        #y
        , sy()
        , mdap(1)
        , vt2(53,[51])
        , vtc2(45,[15,48])
        , vstem2(18)
        , v_notch_ko_kai(40,27,21)
        , srp0(1)
        , vbct(36)
        , alignrp(8)
        , head1(30,[54,60,49,3],ss=14,l1s=66,l2s=72) 
        , iy()
        , dh1(11)
        , dh2(33,5)
        ),

    'uni0E0D.descless':    #YO YING
        packline(""
        , mdap(26,"9-8 10-8 11-8 12-8 15-8 ")
        , hstem1(14)
        , mdrprg(11)
        , mdrp0rg(29,"10+8 11+8 12+8 ")
        , hstem1(8)
        , srp0(26)
        , head1(20,[49,43])
        #
        , mdap(39,"9-8 ")
        , hstem2(37)
        , mdrp0rg(35,"17+8 ")
        , hstem2(1)
        , ix()
        , dh1(17)
        , dh2(23)
        #y
        , sy()
        , mdap(1)
        , vstem1(36)
        , vt2(39)
        , vtc2(32,[2,35])
        , vstem2(5)
        , v_notch_ko_kai(27,14,8)
        , srp0(1)
        , vbct(23)
        , head1(17,[40,46])
        , iy()
        , dh2(20)
        ),

    'desc.uni0E0D':
        packline(""
        , mdap(25)
        , mdrp0rg(6)
        , headsara1(12,[39,33])
        , ix()
        #y
        , sy()
        , mdap(42)
        , mdrp0mrw(9)
        , mdrprg(22)
        , headsara2(3,[30,36])
        , mdrp0rg(0)
        , vstemsara1(16)
        , iy()
        ),

    'uni0E0E':    #DO CHADA
        packline(""
        , hf_pho_samphao(40,52,37,55,34,47,[71,65])
        #
        , srp0(40)
        , hw3(61)
        , hstem2(27)
        , shprp2(1)
        , mdrp0rg(21,"13-8 14-8 15-8 16-8 17-8 18-8 ")
        , hstem2(15)
        , srp0(61)
        , mdrp0rg(8)
        , hstem1(81)
        , add_ip(81,21,[13,76,25,3])
        , ix()
        , dh1(50,11)
        , dh2(44,5)
        #y
        , sy()
        , mdap(84)
        , vtc2(58,[28,61])
        , vstem2(31)
        , v_notch_ko_kai(53,40,34)
        , srp0(84)
        , vbct(44)
        , head1(50,[62,68])
        , srp0(84)
        , vblum(1)
        , vstem1(27)
        , mdrp0rg(5)
        , vstem1(74)
        , mdrp0rg(11)
        , f_ip([13,76,25,3])
        , mdrprg(18)
        , vstem2(78)
        , iy()
        , dh1(47,8)
        ),

    'uni0E0F':    #TO PATAK
        packline(""
        , hf_pho_samphao(53,65,50,68,47,60,[84,78])
        #
        , srp0(53)
        , hw3(74)
        , hstem2(40)
        , mdrp0rg(26)
        , f_ip([32,36,6,5,3,1])
        , hstem2(20)
        , srp0(74)
        , mdrp0rg(13)
        , hstem1(94)
        , add_ip(94,26,[18,89,30,8])
        , ix()
        , dh1(63,16)
        , dh2(57,10)
        #y
        , sy()
        , mdap(97)
        , vtc2(71,[41,74])
        , vstem2(44)
        , v_notch_ko_kai(66,53,47)
        , srp0(97)
        , vbct(57)
        , head1(63,[75,81])
        , srp0(97)
        , vblum(10)
        , alignrp(6)
        , alignrp(1)
        , vstem1(87)
        , shprp2(32)
        , shprp2(40)
        , mdrp0rg(16)
        , alignrp(36)
        , f_ip([18,89,30,8])
        , vstem2(91)
        , shprp2(3)
        , iy()
        , dh1(60,13)
        ),

    'uni0E10.descless':    #THO THAN
        packline(""
        , mdap(44,"11+8 ")
        , mdrprg(31)
        , hstem2(17)
        , mdrp0rg(13,"11-8 12-8 ")
        , alignrp(47)
        , hstem2(1)
        , head2(7,[57,51])
        , srp0(44)
        , mdrp0rg(22,"9-8 10-8 11-8 12-8 13-8 15-8 16-8 ")
        , hstem1(40)
        , ix()
        , dh1(10)
        , dh2(4)
        #y
        , sy()
        , mdap(1)
        , vtc2(25,[47,14])
        , vstem2(37)
        , mdrp0rg(28)
        , vstem2(34)
        , srp0(25)
#        , mdrp0rg(20)
#        , vstem1(41)
#        , mdrp0mrg(10,"9+8 10+8 15+8 16+8 17+8 ")
        , mdrp0rg(20,"11-8 12-8 13-8 14-8 ")
        , vstem1(41)
        , mdrp0mrg(10,"9+8 10+8 11+8 12+8 13+8 14+8 15+8 16+8 17+8 ")
        , head2(4,[48,54])
        , iy()
        , dh1(7)
        ),

    'desc.uni0E10':
        packline(""
        , mdap(40)
        , hstemsara2(28)
        , headsara2(34,[50,44])
        , mdrp0rg(23)
        , f_ip([24,25,26,1,2,3])
        , headsara2(11,[62,66,17,5])
        , ix()
        #y
        , sy()
        , mdap(65)
        , mdrp0mrw(37)
        , headsara2(31,[41,47])
        , mdrp0rg(1)
        , alignrp(4)
        , alignrp(8)
        , vstemsara1(27)
        , shprp2(24)
        , headsara1(14,[53,59])
        , iy()
        ),

    'uni0E11':    #THO NANGMONTHO
        packline(""
        , hf_kho_khuat(38,27, 32, [71,65], dstr1="")
        , hr_tho_nangmontho(61,1, 13,11,52, 46,19, 55,8)
        , add_ip(38,46, [41,42,43, 22,23,24])
        , ix()
        , dh1(23)
        , dh2(35)
        #y
        , sy()
        , mdap(1)
        , alignrp(12)
        , vtc2(43,[19,46,14,51,52,11])
        , alignrp(58)
        , alignrp(41)
        , vstem2(22)
        , shprp2(24)
        , shprp2(5)
        , mdrp0rg(23)
        , vstem1(42)
        , mdrp0rg(29)
        , head2(35,[62,68],small=True)
        , iy()
        , dh2(32)
        ),

    'uni0E12':    #THO PHUTHAO
        packline(""
        , mdap(49,"9-8 10-8 11-8 12-8 15-8 24-8 ")
        , hstem1(48)
        , f_ip([30])
        , mdrp0rg(55,"11+8 12+8 ")
        , hstem1(25)
        , srp0(49)
        , mdrp0rg(36,"9+8 10+8 11+8 13-8 15-8 16-8 17-8 ")
        , head1(42,[78,72,33,45],small=True)
        #
        , mdap(68)
        , hstem2(66)
        , shprp2(1)
        , mdrp0rg(63,"11-8 12-8 21+8 ")
        , hstem2(16)
        , mdrp0rg(11,"32+8 ")
        , head1(5,[90,84])
        , add_ip(35,63,[58,59,60,20,21,22])
        , ix()
        , dh1(39,14)
        , dh2(8)
        #y
        , sy()
        , mdap(1)
        , alignrp(48)
        , vstem1(66)
        , shprp2(30)
        , vt2(68)
        , vtc2(60,[55,25,17,63])
        , alignrp(58)
        , vstem2(20)
        , shprp2(22)
        , mdrp0rg(21,"13-8 14-8 15-8 ")
        , vstem1(59)
        , srp0(60)
        , mdrp0mrw(39,"9-8 10-8 11-8 ")
        , head2(45,[69,75,33],small=True)
        , srp0(1)
        , vbct(8)
        , head1(14,[81,87,16,64,3])
        , iy()
        , dh1(36,11)
        , dh2(42,5)
        ),

    'uni0E13':    #NO NEN
        packline(""
        , mdap(26,"9-8 10-8 11-8 15-8 16-8 17-8 18-8 19-8 ")
        , hstem1(14)
        , mdrprg(11)
        , mdrp0rg(29,"10+8 11+8 ")
        , hstem1(8)
        , srp0(26)
        , head1(20,[61,55])
        #
        , mdap(40,"9-8 17-8 19-8 29-8 ")
        , hstem2(38)
        , mdrp0rg(44,"27-8 ")
        , head2(50,[73,67])
        , srp0(40)
        , mdrp0rg(35,"11-8 18-8 ")
        , alignrp(0)
        , hstem2(1)
        , ix()
        , dh1(17)
        , dh2(23,47)
        #y  
        , sy()
        , mdap(1)
        , vt2(40)
        , vtc2(32,[2,35,36])
        , vstem2(5)
        , v_notch_ko_kai(27,14,8)
        , srp0(1)
        , vbct(23)
        , alignrp(47)
        , head1(17,[52,58,41,50], ss=38,l1s=64,l2s=70)
        , iy()
        , dh2(20,44)
        ),

    'uni0E14':    #DO DEK
        packline(""
        , h_kho_khwai(32,31, 38,8,13, 19,25,[54,48,16,28], 44,1, [41,5])
        , ix()
        , dh1(22)
        #y
        , sy()
        , mdap(1)
        , alignrp(31)
        , vstem1(13)
        , vtc2(41)
        , vstem2(5)
        , mdrp0rw(22,"9-8 10-8 ")
        , head2(28,[45,51,16],small=True)
        , iy()
        , dh1(19)
        , dh2(25)
        ),

    'uni0E15':    #TO TAO
        packline(""
        , h_kho_khwai(34,33, 40,10,15, 21,27,[58,52,18,30], 48,1, [43,44,45,5,6,7])
        , ix()
        , dh1(24)
        #y
        , sy()
        , mdap(1)
        , alignrp(33)
        , vstem1(15)
        , vtc2(45)
        , alignrp(43)
        , vstem2(5)
        , shprp2(7)
        , mdrp0rg(6,"11-8 12-8 13-8 14-8 15-8 18-8 ")
        , vstem1(44)
        , srp0(45)
        , mdrp0rw(24,"9-8 10-8 ")
        , head2(30,[49,55,18],small=True)
        , iy()
        , dh1(21)
        , dh2(27)
        ),

    'uni0E16':      #THO THUNG
        packline(""
        , hf_ko_kai(26,14,11, 29,8)
        , srp0(26)
        , head1(20,[45,39])
        , srp0(26)
        , hw1(35)
        , hstem2(1)
        , ix()
        , dh1(17)
        , dh2(23)
        #y
        , sy()
        , mdap(1)
        , vtc2(32)
        , vstem2(5)
        , v_notch_ko_kai(27,14,8)
        , srp0(1)
        , vbct(23)
        , head1(17,[36,42])
        , iy()
        , dh2(20)
        ),

    'uni0E17':      #THO THAHAN
        packline(""
        , hf1(24,12,18,[43,37],s1s=10)
        , srp0(24)
        , hw3(33,[8])
        , hstem2(1)
        , mdap(8)
        , hstem2(27)
        , ix()
        , dh1(21)
        , dh2(15)
        #y
        , sy()
        , mdap(1)
        , alignrp(11)
        , vtc2(21,[10,25])
        , alignrp(30)
        , vstem2(5)
        , head2(15,[34,40])
        , iy()
        , dh1(18)
        ),

    'uni0E18':      #THO THONG
        packline(""
        , mdap(34)
        , hstem2(5)
        , mdrprg(21,"12-8 ")
        , mdrp0rg(1,"15-8 22-8 25-8 ")
        , hstem1(3)
        , srp0(34)
        , mdrp0rg(11,"9-8 12-8 15-8 ")
        , hstem1(30)
        , ix()
        , dh1(18)
        , dh2(24)
        #y
        , sy()
        , mdap(1)
        , vstem1(4)
        , vtc2(14)
        , vstem2(27)
        , mdrp0rg(24)
        , vstem1(17)
        , srp0(14)
        , mdrp0rg(9)
        , vstem1(31)
        , mdrpg(3)
        , iy()
        ),

    'uni0E19':      #NO NU
        packline(""
        , hf1(21,9,15,[43,37],s1s=8,dstr1="12-8 ")
        , srp0(21)
        , mdrp0rg(27,"10-8 14-8 ")
        , hstem2(25)
        , mdrp0rg(31)
        , head2(3,[55,49])
        , ix()
        , dh1(18)
        , dh2(12,0)
        #y
        , sy()
        , mdap(8)
        , vstem1(22)
        , vt2(27)
        , vtc2(18)
        , head2(12,[34,40])
        , srp0(8)
        , vbct(0)
        , head1(25,[46,52,28,5])
        , iy()
        , dh1(15,3)
        , dh2(31)
        ),  

    'uni0E1A':      #BO BAIMAI
        packline(""
        , hf1(18,6,12,[32,26],add1=mdrprg(3))
        , srp0(18)
        , hw3(22)
        , hstem2(20)
        , ix()
        #y  
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(19)
        , vt2(22)
        , vtc2(15)
        , head2(9,[23,29])
        , iy()
        ),

    'uni0E1B':      #PO PLA
        packline(""
        , hf1(18,6,12,[32,26],add1=mdrprg(3))
        , srp0(18)
        , hw3(22)
        , hstem2(20)
        , ix()
        , dh1(15)
        , dh2(9)
        #y  
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(19)
        , vtpo(22)
        , vtc2(15)
        , head2(9,[23,29])
        , iy()
        , dh1(12)
        ),

    'uni0E1C':      #PHO PHUNG
        packline(""
        , mdap(4,"9-8 13-8 14-8 15-8 16-8 17-8 ")
        , hstem1(16)
        , shprp2(3)
        , head1(11,[32,26])
        , srp0(4)
        , mdrp0rg(22,"9+8 11+8 13+8 14+8 15+8 16+8 17+8 ")
        , f_ip([18,19,2])
        , hstem2(20)
        , shprp2(1)
        , ix()
        , dh1(8)
        , dh2(14)
        #y  
        , sy()
        , mdap(1)
        , alignrp(3)
        , vt2(22)
        , vtc2(8,[17,20])
        , head2(14,[23,29])
        , add_ip(14,1,[18,2])
        , iy()
        , dh2(11)
        ),  

    'uni0E1D':      #FO FA
        packline(""
        , mdap(4,"9-8 13-8 14-8 15-8 16-8 17-8 ")
        , hstem1(16)
        , shprp2(3)
        , head1(11,[32,26])
        , srp0(4)
        , mdrp0rg(22,"9+8 11+8 13+8 14+8 15+8 16+8 17+8 ")
        , f_ip([18,19,2])
        , hstem2(20)
        , shprp2(1)
        , ix()
        , dh1(8)
        , dh2(14)
        #y  
        , sy()
        , mdap(1)
        , alignrp(3)
        , vtpo(22)
        , vtc2(8,[17,20])
        , head2(14,[23,29])
        , add_ip(14,1,[18,2])
        , iy()
        , dh2(11)
        ),  

    'uni0E1E':      #PHO PHAN
        packline(""
        , hf1(18,6,12,[34,28],s1s=4)
        , srp0(18)
        , hw3(24)
        , hstem2(22)
        , shprp2(2)
        , add_ip(18,22,[20,21,3])
        , ix()
        , dh1(15)
        , dh2(9)
        #y
        , sy()
        , mdap(1)
        , alignrp(5)
        , vt2(24)
        , shprp2(21)
        , vtc2(15,[3,19,22,4,2])
        , head2(9,[25,31])
        , iy()
        , dh1(12)
        ),

    'uni0E1F':      #FO FAN
        packline(""
        , hf1(18,6,12,[34,28],s1s=4)
        , srp0(18)
        , hw3(24)
        , hstem2(22)
        , shprp2(2)
        , add_ip(18,22,[20,21,3])
        , ix()
        , dh1(15)
        , dh2(9)
        #y
        , sy()
        , mdap(1)
        , alignrp(5)
        , vtpo(24)
        , vt2(21)
        , vtc2(15,[3,19,22,4,2])
        , head2(9,[25,31])
        , iy()
        , dh1(12)
        ),

    'uni0E20':    #PHO SAMPHAO
        packline(""
        , hf_pho_samphao(14,26,11,29,8,21,[45,39])
        #
        , srp0(14)
        , hw3(35)
        , hstem2(1)
        , ix()
        , dh1(24)
        , dh2(18)
        #y
        , sy()
        , mdap(1)
        , vtc2(32,[2,35])
        , vstem2(5)
        , v_notch_ko_kai(27,14,8)
        , srp0(1)
        , vbct(18)
        , head1(24,[36,42])
        , iy()
        , dh1(21)
        ),

    'uni0E21':      #MO MA
        packline(""
        , hf1(27,15,21,[42,36],s1s=3,s2s=45,s3s=10,l1s=49,l2s=54)
        , srp0(27)
        , hw3(32)
        , hstem2(30)
        , shprp2(1)
        , ix()
        , dh1(24,13)
        , dh2(18,9)
        #y
        , sy()
        , mdap(1)
        , vstem1(30)
        , vt2(32)
        , vtc2(24)
        , head2(18,[33,39])
        , srp0(1)
        , vbct(7)
        , head1(13,[45,51,15,49,28,3])
        , iy()
        , dh1(21,10)
        ),
    
    'uni0E22':    #YO YAK
        packline(""
        , mdap(1)
        , hstem1(34)
        , f_ip([5])
        , mdrprg(28)
        , mdrp0rg(22,"13+8 14+8 15+8 16+8 17+8 ")
        , hstem2(8)
        , mdrp0rg(20,"13-8 14-8 15-8 16-8 17-8 ")
        , head1(14,[48,42])
        , srp0(1)
        , hw1(38)
        , hstem2(36)
        , ix()
        , dh1(11)
        , dh2(17)
        #y
        , sy()
        , mdap(1)
        , vstem1(35)
        , vt2(38)
        , vtc2(11,[5,22])
        , head2(17,[39,45,20],small=True)
        , mdrp0mrg(31,"13-8 14-8 ")
        , vstem1(25)
        , f_ip([5])
        , iy()
        , dh2(14)
        ),

    'uni0E23':    #RO RUA
        packline(""
        , mdap(36,"9-8 ")
        , hstem2(8)
        , mdrprg(23)
        , head2(3,[49,43])
        , srp0(36)
        , mdrp0rg(14)
        , hstem1(32)
        , ix() 
        , dh1(6)
        , dh2(0)
        #y
        , sy() 
        , mdap(52)
        , vtc2(17)
        , vstem2(29)
        , mdrp0rg(26)
        , vstem1(20)
        , srp0(17)
        , mdrp0rg(12)
        , vstem1(33)
        , srp0(52)
        , vbct(0)
        , head1(6,[40,46])
        , iy()
        , dh1(3)
        ),  

    'uni0E24':      #RU
        packline(""
        , hf_ko_kai(26,14,11, 29,8)
        , srp0(26)
        , head1(20,[45,39])
        , srp0(26)
        , hw1(35)
        , hstem2(1)
        , ix()
        , dh1(17)
        , dh2(23)
        #y
        , sy()
        , mdap(48)
        , vblu(0)
        , vtc2(32)
        , vstem2(5)
        , v_notch_ko_kai(27,14,8)
        , srp0(48)
        , vbct(23)
        , head1(17,[36,42])
        , iy()
        , dh2(20)
        ),

    'uni0E25':      #LO LING
        packline(""
        , mdap(19,"9-8 ")
        , hstem1(7)
        , mdrp0rg(38,"9+8 10+8 ")
        , hstem1(33)
        , srp0(19)
        , head1(13,[54,48])
        , srp0(19)
        , hw1(44)
        , hstem2(26)
        , shprp2(1)
        , ix()
        , dh1(10)
        , dh2(16)
        #y
        , sy()
        , mdap(1)
        , vtc2(41,[26])
        , vstem2(30)
        , mdrprg(36,"9+8 10+8 15+8 16+8 17+8 ")
        , mdrp0rg(23,"9-8 10-8 ")
        , vstem2(4)
        , srp0(1)
        , vbct(16)
        , head1(10,[45,51])
        , iy()
        , dh2(13)
        ),

    'uni0E26':    #LU
        packline(""
        , hf_pho_samphao(14,26,11,29,8,21,[45,39])
        #
        , srp0(14)
        , hw3(35)
        , hstem2(1)
        , ix()
        , dh1(24)
        , dh2(18)
        #y
        , sy()
        , mdap(48)
        , vtc2(32,[2,35])
        , vstem2(5)
        , v_notch_ko_kai(27,14,8)
        , srp0(48)
        , vblu(1)
        , vbct(18)
        , head1(24,[36,42])
        , iy()
        , dh1(21)
        ),

    'uni0E27':    #WO WAEN
        packline(""
        , mdap(27)
        , hstem2(8)
        , head2(3,[40,34])
        , srp0(27)
        , mdrp0rg(21,"14-8 ")
        , hstem1(15)
        , ix()
        , dh1(6)
        , dh2(0)
        #y
        , sy()
        , mdap(43)
        , vtc2(24,[9,27])
        , vstem2(12)
        , mdrprg(19)
        , srp0(43)
        , vbct(0)
        , head1(6,[31,37])
        , iy()
        , dh1(3)
        ),  

    'uni0E28':    #SO SALA
        packline(""
        , h_kho_khwai(20,19, 26,56,59, 61,10,[77,71,16,65,8], 49,1,[29,53,32,46,5,50])
        , mdrp0rg(41)
        , hstem2(35)
        , ix()
        , dx(50,4)
        , dx(46,-4)
        , dh1(63)
        , dh2(13)
        #y
        , sy()
        , mdap(1)
        , alignrp(19)
        , vtc2(29)
        , mdrpmrg(38)
        , vstem2(53)
        , mdrp0rw(63,"9-8 10-8 ")
        , f_ip([50,32,46,5])
        , head2(13,[68,74,16,65,8],small=True)
        , dx(16,12)
        , dx(59,-16)
        , iy()
        , dx(32,-4)
        , dx(5,4)
        , dh1(10)
        ),

    'uni0E29':      #SO RUSI
        packline(""
        , hf1(18,6,12,[66,60],add1=pl("" 
            , mdrprg(3)
            , mdrp0mrg(29,"9-8 ")
            , head1(35,[78,72],small=True)
            , srp0(18))
            )
        , srp0(18)
        , hw3(43)
        , alignrp(56)
        , hstem2(41)
        , shprp2(20)
        , mdrprg(52)
        , f_dstr(35,"17-4 18-4 19-4 ")
        , ix()
        , dh1(15,32)
        , dh2(9)
        #y  
        , sy()
        , mdap(1)
        , vstem1(5)
        , shprp2(19)
        , vt2(43)
        , vtc2(15)
        , head2(9,[57,63])
        , mdrp0rg(32, "10+8 15+8 20-8 21-8 ")
        , head2(26,[69,75,29,78,72,35],small=True)
        , mdrp0rg(23)
        , vstem1(39)
        , mdrprg(49)
        , f_ip([41,21,44,36])
        , iy()
        , dh1(12,29)
        , dh2(35)
        ),

    'uni0E2A':      #SO SUA
        packline(""
        , mdap(19,"9-8 ")
        , hstem1(7)
        , mdrp0rg(38,"9+8 10+8 ")
        , hstem1(33)
        , srp0(19)
        , head1(13,[78,72])
        , srp0(19)
        , hw1(59)
        , f_ip([25,66,28,43,57,63])
        , hstem2(60)
        , shprp2(1)
        , mdrp0rg(52)
        , hstem2(46)
        , ix()
        , dx(28,4)
        , dx(57,-4)
        , dh1(10)
        , dh2(16)
        #y
        , sy()
        , mdap(1)
        , vtc2(41,[60,61,59])
        , vstem2(30)
        , mdrpmrg(49)
        , mdrprg(36,"9+8 10+8 15+8 16+8 17+8 ")
        , mdrp0rg(23,"9-8 10-8 ")
        , f_ip([28,43,57,63])
        , vstem2(4)
        , f_ip([25,66])
        , srp0(1)
        , vbct(16)
        , head1(10,[69,75])
        , iy()
        , dx(63,4)
        , dx(43,-4)
        , dh2(13)
        ),

    'uni0E2B':      #HO HIP
        packline(""
        , hf1(24,12,18,[55,49],s1s=10)
        , srp0(24)
        , hw3(45,[8])
        , alignrp(39)
        , hstem2(1)
        , head2(33,[67,61,30,42,58,5])
        , mdap(8)
        , hstem2(27)
        , ix()
        , dh1(21,36)
        , dh2(15)
        #y
        , sy()
        , mdap(1)
        , alignrp(11)
        , vtc2(21,[10,25])
        , alignrp(36)
        , head2(15,[46,52,30,42],ss=5,l1s=64,l2s=58)
        , iy()
        , dh1(18,33)
        , dh2(39)
        ),

    'uni0E2C':      #LO CHULA
        packline(""
        , hf1(18,6,12,[57,51],s1s=4)
        , srp0(18)
        , hw3(47)
        , alignrp(34)
        , hstem2(22)
        , shprp2(2)
        , mdrprg(43)
        , head2(28,[69,63])
        , add_ip(18,22,[20,21,3])
        , ix()
        , dh1(15,31)
        , dh2(9,25)
        #y
        , sy()
        , mdap(1)
        , alignrp(5)
        , vt2m(25)
        , mdrprg(40)
        , f_ip([62,23,35,47])
        , head1(31,[60,66])
        , srp0(1)
        , vtc2(15,[20,3,19,22,4,2])
        , head2(9,[48,54])
        , iy()
        , dh1(12,28)
        ),

    'uni0E2D':      #O ANG
        packline(""
        , mdap(1,"16-8 ")
        , hstem1(13)
        , mdrp0rg(27)
        , hstem1(22)
        , srp0(1)
        , head1(8,[43,37])
        #
        , mdap(33,"9-8 ")
        , hstem2(15)
        , ix()
        , dh1(5)
        , dh2(11)
        #y
        , sy()
        , mdap(1)
        , vstem1(14)
        , vtc2(30,[16,33])
        , vstem2(19)
        , mdrprg(25,"15+8 16+8 17+8 ")
        , mdrp0rg(5,"9-8 10-8 ")
        , head2(11,[34,40])
        , iy()
        , dh2(8)
        ),  

    'uni0E2E':      #HO NOKHUK
        packline(""
        , mdap(1,"16-8 ")
        , hstem1(13)
        , mdrp0rg(24)
        , hstem1(54)
        , srp0(1)
        , head1(8,[66,60])
        #
        , mdap(44,"9-8 ")
        , hstem2(15)
        , mdrprg(38)
        , add_ip(24,44,[48,30,42,18])
        , ix()  
        , dx(40,4)
        , dx(42,-4)
        , dh1(5)
        , dh2(11)
        #y  
        , sy()
        , mdap(1) 
        , vstem1(14)
        , vtc2(27)
        , vstem2(51)
        , mdrprg(35)
        , mdrp0rg(21,"10+8 ")
        , f_ip([48,30,42,18])
        , vstem1(45)
        , mdrp0rg(5,"9-8 10-8 11-8 12-8 13-8 14-8 15-8 16-8 ")
        , head2(11,[57,63])
        , iy()
        , dx(18,4)
        , dx(30,-4)
        , dh2(8)
        ),  

    'uni0E2F':      #PAIYAN NOI
        packline(""
        , mdap(30,"9-8 ")
        , hstem2(29)
        , shprp2(7)
        , mdrprg(3)
        , mdrp0rg(16)
        , head1(22,[40,34])
        , ix()  
        , dh1(19)
        #y  
        , sy()
        , mdap(1)
        , vstem1(5)
        , vtc2(19,[7])
        , alignrp(30)
        , head2(13,[31,37])
        , mdrp0rg(10)
        , vstem1(26)
        , iy()
        , dh2(22)
        ),  

    'uni0E30':      #SARA A
        packline(""
        , mdap(6)
        , alignrp(48)
        , head1(12,[39,33],ss=54,l1s=81,l2s=75)
        , srp0(6)
        , mdrp0rg(25,"10+8 15+8 ")
        , alignrp(67)
        , hstem2(19)
        , shprp2(61)
        , ix()
        , dh1(51,9)
        #y  
        , sy()
        , mdap(0)
        , vstem1(16)
        , mdrp0rg(3)
        , head1(9,[30,36])
        , mdrprg(22)
        , srp0(0)
        , vt2m(51)
        , mdrprg(64)
        , head2(45,[72,78])
        , mdrp0rg(42)
        , vstem1(58)
        , iy()
        , dh2(54,12)
        ),

    'uni0E31':      #MAI HAN-AKAT
        packline(""
        , mdap(6)
        , head1(12,[39,33])
        , srp0(6)
        , mdrp0rg(25,"10+8 15+8 ")
        , hstem2(19)
        , ix()
        , dh1(9)
        #y  
        , sy()
        , mdap(0)
        , vstem1(16)
        , mdrp0rg(3)
        , head1(9,[30,36])
        , mdrprg(22)
        , iy()
        , dh2(12)
        ),

    'uni0E32':    #SARA AA
        packline(""
        , mdap(19)
        , hstem2(1)
        , mdrp0rg(13,"14-8 ")
        , hstem1(8)
        , ix()
        #y
        , sy()
        , mdap(1)
        , vtc2(16,[2,19])
        , vstem2(5)
        , mdrprg(11)
        , iy()
        ),

    'uni0E34':    #SARA I
        packline(""
        , mdap(0)
        , hstem2(7)
        , mdrp0rg(1)
        , hstem1(13)
        , ix()
        #y
        , sy()
        , mdap(0)
        , vstem1(7)
        , mdrp0rg(4,"16+8 17+8 18+8 19+8 ")
        , vstem2(10)
        , iy()
        ),
    
    'uni0E35':    #SARA II
        packline(""
        , mdap(0)
        , hstem2(7)
        , alignrp(10)
        , mdrp0rg(1)
        , hstem1(16)
        , ix()
        #y
        , sy()
        , mdap(0)
        , vstem1(10)
        , mdrp0rg(4,"16+8 17+8 18+8 19+8 ")
        , vstem2(13)
        , mdrpmrg(8)
        , iy()
        ),
    
    'uni0E36':    #SARA UE
        packline(""
        , mdap(0)
        , hstem2(20)
        , mdrp0mrg(15)
        , headsara2(9,[36,30])
        , srp0(0)
        , mdrp0rg(1)
        , hstem1(26)
        , ix()
        #y
        , sy()
        , mdap(0) 
        , vstem1(20)
        , mdrp0rg(4,"16+8 17+8 18+8 19+8 ")
        , vstem2(23)
        , mdrp0mrg(12)
        , headsara2(18,[27,33])
        , iy()
        ),

    'uni0E37':    #SARA UUE
        packline(""
        , mdap(0)
        , hstem2(11)
        , shprp2(14)
        , mdrp0rg(8,"12-8 13-8 14-8 15-8 16-8 17-8 18-8 ")
        , hstem2(6)
        , srp0(0)
        , mdrp0rg(1)
        , hstem1(20)
        , ix()
        #y  
        , sy()
        , mdap(0)
        , vstem1(20)
        , mdrp0rg(4,"16+8 17+8 18+8 19+8 ")
        , f_ip([6,9,11])
        , vstem2(17)
        , mdrpmrg(8)
        , shprp2(13)
        , iy()
        ),  

    'uni0E38':    #SARA U
        packline(""
        , mdap(13)
        , hstem2(1)
        , headsara2(7,[23,17])
        , ix()
        #y  
        , sy()
        , mdap(10) 
        , headsara2(4,[14,20])
        , mdrpmrg(1)
        , iy()
        ),  

    'uni0E39':    #SARA UU
        packline(""
        , mdap(17)
        , hstemsara2m(15)
        , mdrp0mrg(13)
        , mdap(13,"9-8 10-8 ")
        , hstemsara2(1)
        , headsara2(7,[27,21])
        , ix() 
        #y  
        , sy()
        , mdap(10)
        , alignrp(16)
        , headsara2(4,[18,24])
        , mdrp0mrg(1)
        , mdap(1)
        , vstemsara1(14)
        , iy()
        ),

    'uni0E3A':    #PHINTHU
        packline(""
        , mdap(9)
        , mdrpmrg(3)
        , ix() 
        #y  
        , sy()
        , mdap(0)
        , mdrpmrg(6)
        , iy() 
        ),

    'uni0E3F':    #THAI CURRENCY SYMBOL BAHT
        packline(""
        , mdap(9,"11-8 12-8 13-8 14-8 ")
        , hstem1(54)
        , shprp2(50)
        , mdrprg(6)
        , shprp2(13)
        #
        , mdrp0rg(52,"11+8 ")
        , alignrp(48)
        , alignrp(2)
        , alignrp(16)
        , hstem1(64)
        , shprp2(47)
        , shprp2(36)
        , shprp2(21)
        #
        , mdrp0rg(60)
        , f_ip([29])
        , hstem1(32)
        , mdrp0rg(43,"12+8 ")
        , hstem1(26)
        , ix() 
        #y  
        , sy()
        , mdap(3)
        , alignrp(36)
        , mdrprg(0)
        , vstem1(52)
        , shprp2(56)
        , shprp2(9)
        #
        , mdrp0rg(53)
        , f_ip([60,32])
        , alignrp(64)
        , vstem1(48)
        , f_ip([29])
        , shprp2(39)
        #
        , mdrp0rg(16)
        , f_ip([43,26])
        , alignrp(22)
        , mdrprg(19)
        , vstem2(49)
        , shprp2(10)
        , shprp2(47)
        , iy() 
        ), 

    'uni0E40':    #SARA E
        packline(""
        , mdap(3)
        , hstem1(5)
        , head1(11,[23,17])
        , ix()
        , dh1(8)
        , dh2(0)
        #y  
        , sy()
        , mdap(26)
        , vt2(5)
        , vbct(0)
        , head1(8,[14,20])
        , iy()
        , dh2(11)
        ),

    'uni0E42':    #SARA O
        packline(""
        , mdap(3)
        , hstem1(31)
        , head1(37,[49,43])
        , srp0(3)
        , mdrp0rg(9)
        , hstem1(27)
        , mdrprg(18)
        , ix()
        , dh1(34)
        , dh2(0)
        #y  
        , sy()
        , mdap(52)
        , vbct(0)
        , head1(34,[40,46])
        #
        , mdap(12)
        , vstem2(24)
        , mdrp0rg(21)
        , vstem1(15)
        , srp0(12)
        , mdrp0rg(7)
        , vstem1(28)
        , iy()   
        , dh2(37)
        ),

    'uni0E43':    #SARA AI MAI MUAN
        packline(""
        , mdap(3)
        , hstem1(27)
        , head1(33,[45,39])
        , srp0(3)
        , mdrp0rg(21)
        , hstem1(10)
        , head1(15,[57,51])
        , ix()
        , dh1(12,30)
        , dh2(18,0)
        #y  
        , sy()
        , mdap(60)
        , vbct(0) 
        , head1(30,[36,42])
        #
        , mdap(24)
        , vstem2(7)
        , mdrp0rg(12)
        , head2(18,[48,54])
        , iy()
        , dh2(15,33)
        ),

    'uni0E44':    #SARA AI MAIMALAI
        packline(""
        , mdap(3)
        , hstem1(22)
        , head1(28,[40,34])
        , srp0(3)
        , mdrp0rg(12)
        , f_ip([5,6,20,21])
        , hstem1(17)
        , ix()
        , dh1(25)
        , dh2(0)
        #y  
        , sy()
        , mdap(43)
        , vbct(0)
        , head1(25,[31,37])
        #
        , mdap(22)
        , vstem2(4)
        , mdrprg(15)
        , mdrp0rg(5)
        , vstem1(20)
        , iy()
        , dh2(28)
        ),

    'uni0E45':    #LAKKHANGYAO
        packline(""
        , mdap(19)
        , hstem2(1)
        , mdrp0rg(13,"14-8 ")
        , hstem1(8)
        , ix()
        #y
        , sy()
        , mdap(20)
        , vblu(1)
        , vtc2(16,[2,19])
        , vstem2(5)
        , mdrprg(11)
        , iy()
        ),

    'uni0E46':    #MAIYAMOK
        packline(""
        , mdap(37)
        , hstem2(9)
        , mdrprg(3)
        , mdrp0rg(29)
        , f_ip([32,33,34,13,14,15])
        , hstem1(18)
        , head1(23,[50,44])
        , ix()
        , dh1(20)
        , dh2(26)
        #y
        , sy()
        , mdap(53)
        , vtc2(34,[10,37])
        , alignrp(32)
        , vstem2(13)
        , shprp2(15)
        , mdrp0rg(14,"12-8 13-8 14-8 ")
        , vstem1(33)
        , srp0(34)
        , mdrp0rg(20)
        , head2(26,[41,47])
        #
        , srp0(53)
        , vblum(0)
        , vstem1(6)
        , iy()
        , dh2(23)
        ),

    'uni0E47':    #MAITAIKHU
        packline(""
        , mdap(42)
        , mdrprg(18)
        , headsara2(36,[55,49])
        , mdrp0rg(7)
        , f_ip([30,31,5])
        , hstemsara1(26)
        , ix()
        #y
        , sy()
        , mdap(0)
        , alignrp(4)
        , vstemsara1(29)
        , headsara1(39,[45,52,31,3,34])
        , mdrp0mrg(22)
        , vstemsara1(11)
        , mdrpmrg(15,"13+4 14+4 15+4 16+4 17+4 ")
        , iy()
        ),

    'uni0E48.low':    #MAI EK
        packline(""
        , mdap(7)
        , vstem2(2)
        , ix()
        #y
        , sy()
        , mdap(0)
        , mdrpmrg(5)
        , iy()
        ),

    'uni0E49.low':    #MAI THO
        packline(""
        , mdap(15)
        , mdrprg(26)
        , headsara2(9,[39,33,4,18])
        , mdrprg(0)
        , ix()
        #y
        , sy()
        , mdap(0)
        , vstemsara1(1)
        , shprp2(18)
        , mdrp0mrg(6)
        , headsara1(12,[30,36])
        , mdrpmrg(23)
        , f_dstr(6,"13+8 14+8 15+8 16+8 17+8 18+8 ")
        , iy()
        ),

    'uni0E4A.low':    #MAI TRI
        packline(""
        , mdap(59)
        , hstemsara2(52)
        , mdrp0rg(48)
        , hstemsara2(8)
        , mdrp0rg(15)
        , hstemsara2(17)
        , mdrp0rg(29)
        , headsara2(35,[72,66,24])
        , ix()
        #y
        , sy()
        , mdap(1)
        , alignrp(32)
        , vstemsara1(7)
        , shprp2(49)
        , headsara1(26,[63,69,17])
        , mdrp0rg(39)
        , alignrp(45)
        , f_ip([42])
        , mdrpmrg(56)
        , vstemsara2(21)
        , shprp2(12)
        , iy()
        ),

    'uni0E4B.low':    #MAI CHATTAWA
        packline(""
        , mdap(18)
        , mdrp0rg(21)
        , alignrp(14)
        , hstemsara2m(2)
        , alignrp(9)
        , mdrprg(6)
        , ix()
        #y  
        , sy()
        , mdap(0)
        , mdrp0rg(21)
        , alignrp(3)
        , vstemsara1m(15)
        , alignrp(9)
        , mdrprg(12)
        , iy()
        ),

    'uni0E4C.low':    #THANTHAKAT
        packline(""
        , mdap(22)
        , mdrprg(16)
        , headsara2(3,[34,28])
        , ix()
        #y  
        , sy()
        , mdap(0)
        , headsara1(6,[25,31])
        , mdrpmrg(13)
        , vstemsara2(20)
        , iy()
        ),

    'uni0E4D':    #NIKHAHIT
        packline(""
        , mdap(9)
        , head2(3,[21,15])
        , ix()
        #y
        , sy()
        , mdap(0)
        , head1(6,[12,18])
        , iy()
        ),

    'uni0E4E':    #YAMAKKAN
        packline(""
        , mdap(18)
        , mdrp0rg(10)
        , f_ip([36,8,55])
        , hstemsara1(30)
        , mdrp0rg(3)
        , hstemsara1(48)
        , ix()
        #
        , sy()
        , mdap(0)
        , vstemsara1(51)
        , mdrp0rg(6)
        , f_ip([39])
        , vstemsara2(45)
        , mdrp0rg(13)
        , f_ip([21])
        , vstemsara2(27)
        , iy()
        ),
#        packline(""
#        , mdap(18)
#        , hstemsara2(24)
#        , mdrp0rg(10)
#        , f_ip([13,27])
#        , hstemsara1(30)
#        , f_ip([8])
#        , srp0(18)
#        , mdrp0mrw(36)
#        , hstemsara2(42)
#        , mdrpmrw(55)
#        , mdrp0rg(3)
#        , f_ip([6,45])
#        , hstemsara1(48)
#        , ix()
#        #y
#        , sy()
#        , mdap(0)
#        , vstemsara1(51)
#        , mdrp0rg(6)
#        , f_ip([3,48])
#        , vstemsara2(45)
#        , f_ip([8])
#        , srp0(0)
#        , mdrp0rg(39)
#        , vstemsara1(33)
#        , mdrp0rg(13)
#        , f_ip([10,30])
#        , vstemsara2(27)
#        , srp0(33)
#        , mdrp0mrw(21)
#        , vstemsara1(16)
#        , iy()
#        ),

    'uni0E4F':    #FONGMAN
        packline(""
        , mdap(3) 
        , hstem1(21)
        , mdrprg(27)
        , mdrp0rg(9)
        , hstem2(15)
        , mdrprg(33)
        , add_ip(27,33,[45,39])
        , ix()
        #y  
        , sy()
        , mdap(6) 
        , vstem2(18)
        , mdrprg(30)
        , mdrp0rg(0)
        , vstem1(12)
        , mdrprg(24)
        , add_ip(30,24,[36,42])
        , iy()
        ),

    'uni0E50':    #THAI DIGIT ZERO
        packline(""
        , mdap(3,"9-8 ")
        , hstem1(21)
        , mdrp0rg(9)
        , hstem2(15)
        , ix()
        #y  
        , sy()
        , mdap(24)
        , vtnc(6)
        , vstem2(18)
        , srp0(24)
        , vbct(0)
        , vstem1(12)
        , iy()
        ),

    'uni0E51':    #THAI DIGIT ONE
        packline(""
        , mdap(3,"9-8 10-8 11-8 15-8 16-8 17-8 ")
        , hstem1(29)
        #
        , mdap(9,"15+8 ")
        , hstem2(23)
        , mdrprg(17)
        , mdrp0rg(44,"9-8 10-8 11-8 ")
        , head2(38,[56,50,34])
        , ix()
        , dh1(41)
        #y  
        , sy()
        , mdap(59)
        , mdrpmrg(14)
        , vtnc(6)
        , vstem2(26)
        , srp0(59)
        , mdrp0rg(0)
        , vstem1(32)
        , mdrp0rg(34,"10-8 11-8 12-8 13-8 ")
        , head1(41,[47,53])
        , iy()
        , dh1(38)
        , dh2(44)
        ),

    'uni0E52':    #THAI DIGIT TWO
        packline(""
        , mdap(1,"9-8 10-8 15-8 16-8 17-8 ")
        , hstem1(17)
        , mdrp0rg(8)
        , hstem1(14)
        , srp0(1)
        , mdrp0rg(42,"13+8 14+8 15+8 16+8 17+8 ")
        , hstem1(31)
        , head1(36,[63,57])
        #
        , mdap(50,"9-8 ")
        , hstem2(22)
        , add_ip(42,50,[45,46,47,26,27,28])
        , ix()
        , dh1(33)
        , dh2(39)
        #y  
        , sy()
        , mdap(0)
        , vstem1(19)
        , vtnc(47)
        , alignrp(45)
        , mdrprg(11)
        , mdrp0rg(27,"13-8 14-8 15-8 16-8 ")
        , vstem1(46)
        , srp0(47)
        , mdrp0rg(33,"14+8 15+8 16+8 17+8 ")
        , head2(39,[54,60])
        , iy()
        , dh2(36)
        ),

    'uni0E53':    #THAI DIGIT THREE
        packline(""
        , mdap(29,"9-8 10-8 17-8 18-8 ")
        , hstem1(17)
        , head1(23,[52,46])
        #
        , mdap(42,"9-8 16+8 ")
        , hstem2(1)
        , mdrp0rg(8,"9+8 15+8 ")
        , hstem2(10)
        , f_ip([36])
        , ix()
        , dh1(20)
        , dh2(26)
        #y  
        , sy()
        , mdap(1)
        , alignrp(10)
        , vtnc(39,[30,17,11,8,30,2,42])
        , alignrp(33)
        , vstem2(5)
        , shprp2(14)
        , srp0(1)
        , vbct(26)
        , head1(20,[43,49])
        , iy()
        , dh2(23)
        ),


    'uni0E54':    #THAI DIGIT FOUR
        packline(""
        , mdap(4,"9-8 10-8 11-8 ")
        , hstem1(23)
        , mdrp0rg(30)
        , head1(36,[58,52,41,27])
        , mdrprg(47)
        , mdrprg(15)
        , ix()
        , dh1(33)
        , dh2(39)
        #y  
        , sy()
        , mdap(1)
        , vstem1(26)
        , shprp2(44)
        , vtn(7)
        , vstem2(20)
        , mdrprg(12)
        , mdrp0rg(34,"10-8 11-8 12-8 13-8 14-8 ")
        , head2(40,[49,55])
        , iy()
        , dh1(30)
        , dh2(36)
        ),

    'uni0E55':    #THAI DIGIT FIVE
        packline(""
        , mdap(4,"9-8 10-8 11-8 ")
        , hstem1(32)
        , mdrp0rg(39)
        , head1(45,[67,61,50,36])
        , mdrprg(56)
        , mdrprg(24)
        , srp0(39)
        , mdrp0rg(9)
        , head1(15,[79,73])
        , ix()
        , dh1(42,12)
        , dh2(48)
        #y  
        , sy()
        , mdap(1)
        , vstem1(35)
        , shprp2(53)
        , vtn(7)
        , alignrp(17)
        , vstem2(29)
        , mdrprg(21)
        , mdrp0rg(42,"10-8 11-8 12-8 13-8 14-8 ")
        , head2(48,[58,64])
        , srp0(29)
        , head1(12,[70,76])
        , iy()
        , dh1(39,9)
        , dh2(45,15)
        ),

    'uni0E56':    #THAI DIGIT SIX
        packline(""
        , mdap(27,"9-8 ")
        , hstem1(43)
        , mdrprg(34)
        , mdrp0rg(6)
        , head1(12,[62,56])
        #
        , mdap(50)
        , hstem2(20)
        , ix()
        , dh1(9)
        #y  
        , sy()
        , mdap(65)
        , vtnc(47) 
        , mdrprg(37)
        , vstem2(23)
        , srp0(65)
        , vbct(0)
        , vstem1(17)
        , mdrp0rg(9, "9-4 ")
        , head2(3,[53,59])
        , iy()
        , dh1(6)
        , dh2(12)
        ),

    'uni0E57':    #THAI DIGIT SEVEN
        packline(""
        , mdap(34,"9-8 10-8 11-8 12-8 13-8 ")
        , hstem1(22)
        , head1(28,[69,63])
        #
        , mdap(56,"9-8 ")
        , hstem2(51)
        , mdrp0rg(47,"13-8 23+8 ")
        , hstem2(6)
        , mdrp0rg(13,"10+8 17+8 22-8 ")
        , hstem2(15)
        , ix()
        , dh1(25)
        , dh2(31)
        #y  
        , sy()
        , mdap(1)
        , vstem1(6)
        , shprp2(48)
        , vtnc(44,[35,22,16,41,13,7,47])
        , alignrp(38)
        , mdrprg(54)
        , vstem2(10)
        , shprp2(19)
        , srp0(1)
        , vbct(31)
        , head1(25,[60,66])
        , mdrpg(15,"15+4 ")
        , iy()
        , dh2(28)
        ),

    'uni0E58':    #THAI DIGIT EIGHT
        packline(""
        , mdap(8,"9-8 10-8 ")
        , hstem1(27)
        , mdrp0rg(31)
        , hstem1(4)
        #
        , mdap(47,"9-8 ")
        , mdrprg(19,"9+8 ")
        , head2(41,[62,56,33])
        , ix()  
        , dh1(44)
        #y  
        , sy()
        , mdap(65)
        , vtn(11)
        , mdrprg(16)
        , vstem2(24)
        , srp0(65)
        , vbct(0)
        , alignrp(5)
        , vstem1(36)
        , shprp2(30)
        , mdrp0rg(32)
        , vstem2(4)
        , mdrp0rg(44,"9+4 15+8 ")
        , head2(50,[53,59])
        , iy()
        , dh1(41)
        , dh2(47)
        ),

    'uni0E59':    #THAI DIGIT NINE
        packline(""
        , mdap(25,"9-8 10-8 ")
        , hstem1(8)
        , mdrp0rg(19)
        , head2(13,[67,61])
        , srp0(19)
        , mdrp0rg(2)
        , hstem1(57)
        , f_ip([31])
        , mdrp0rg(1)
        , hstem1(0)
        , mdrprg(46,"9+8 10+8 ")
        , ix()  
        , dh1(16)
        , dh2(22)
        #y  
        , sy()
        , mdap(1)
        , vtnc(34,[31,2,57])
        , alignrp(28)
        , mdrprg(43)
        , vstem2(54)
        , shprp2(5)
        , mdrp0rg(51)
        , vstem1(37)
        , srp0(1)
        , vbct(22)
        , head1(16,[58,64])
        , iy()
        , dh1(19)
        ),

    'uni0E5A':      #ANGKHANKHU
        packline(""
        , mdap(46,"9-8 ")
        , hstem2(44)
        , mdrp0rg(30,"10-8 11-8 12-8 13-8 14-8 15-8 16-8 17-8 18-8 ")
        , hstem2(29)
        , shprp2(7)
        , mdrprg(3)
        , mdrp0rg(16)
        , head1(22,[40,34])
        , ix()
        , dh1(19)
        #y  
        , sy()
        , mdap(1)
        , alignrp(44)
        , vstem1(5)
        , vtc2(19,[7])
        , alignrp(30)
        , alignrp(46)
        , head2(13,[31,37])
        , mdrp0rg(10)
        , vstem1(26)
        , iy()
        , dh2(22)
        ),

    'uni0E5B':      #KHOMUT
        packline(""
        , mdap(3,"10-8 ")
        , hstem1(26)
        , mdrp0rg(9)
        , hstem2(20)
        , head2(15,[71,65])
        , srp0(9)
        , mdrprg(46)
        , f_ip([32,33,34,35,36,37,38,39,40,41,51,52,53,54,55,56,57,58])
        , ix()
        , dh1(18)
        , dh2(12)
        #y  
        , sy()
        , mdap(74)
        , vtnc(6)
        , vstem2(23)
        , srp0(74)
        , vbct(0)
        , vstem1(29)
        , mdrp0rg(12)
        , alignrp(56)
        , alignrp(53)
        , head1(18,[62,68])
        , alignrp(33)
        , alignrp(36)
        , alignrp(39)
        , mdrp0rg(43)
        , vstem2(49)
        , iy()
        , dh1(15)
        ),

    }



#++++++++++++++++++++++++++++++

if __name__ == "__main__":
    x = sys.argv
    if '--all' in x:    #find&replace in sfd file
        #$0 --all = ENCODE ALL CHARS
        sfd_file = os.path.basename(sys.argv[0]).split('.',1)[0] + '.sfd'
        print "Encoding all character in %s" % (sfd_file)
        dfont.inst_encode_all( sfd_file, pseudo_code_dict, cvt_dict,func_arg_list )
    elif '-b' in x:   #both pseudo code & instructed code
        x.remove('-b')
        txt = pseudo_code_dict[x[1]]
        gedit(txt,1)
    else:           #instructed code
        txt = pseudo_code_dict[x[1]]
        gedit(txt)
